module fkfilter_mod
  
  implicit none

  integer, private  :: pi=3.14159265

contains
  
  subroutine filter1Dfn(temp,mode,flag,alpha)
    !! mode=0 -- do nothing
    !! mode=1 -- sharp cut
    !! mode=2 -- Tukey filter using alpha parameter
    real,dimension(:),intent(inout) :: temp
    real,intent(inout),optional        :: alpha
    integer                         :: mode, flag, n, nn, i

    if( .not.present(alpha) .and. mode.eq.2 ) then
       print *, "WARNING: alpha are not present when using mode2 in filter1Dfn()"
       alpha = 0.3   ! alpha=0.3 is recommended
    end if

    !! temp(n/2+1) is always at zero frequency or wavenumber
    n=size(temp)
    !write(0,*) "n,size(temp),mode,flag",n,size(temp),mode,flag
    !write(0,*) "temp", temp(1), temp(2)

    if(mode.eq.0) then ! do nothing
       temp(1:n) = 1.
    end if

    if(mode.eq.1) then ! step cut
       if(flag.eq.1) then
          ! flag=1: (+)axis will be stored, (-)axis will be removed
          temp(1:n/2) = 0.
          temp(n/2+1:n) = 1.
       else
          ! flag=-1: (-)axis will be stored, (+)axis will be removed.
          temp(1:n/2) = 1.
          temp(n/2+1:n) = 0.
       end if
    end if

    if(mode.eq.2) then ! step cut
       if(alpha .gt. 1.0) then
          print *, "ERROR: alpha must be less than 1 for Tukey window function!!"
          call exit()
       end if

       if(flag.eq.1) then
          ! flag=1: (+)axis will be stored, (-)axis will be removed
          temp(1:n/2) = 0.
          nn=n-int(n/2)-1
          do i=0,nn
             if( i .lt. int(0.5*alpha*float(nn)) )then
                temp(n/2+1+i) = 0.5 + 0.5*cos(pi*(2.0*float(i)/(alpha*float(nn)) - 1.0 ))
             elseif( i.ge.int(0.5*alpha*float(nn)) .and. i.le.(nn-int(0.5*alpha*float(nn))) )then
                temp(n/2+1+i) = 1.
             else
                temp(n/2+1+i) = 0.5 + 0.5*cos(pi*(2.0*float(i)/(alpha*float(nn)) - 2.0/alpha + 1.0 ))
             end if
          end do
       else
          ! flag=-1: (-)axis will be stored, (+)axis will be removed.
          nn=int(n/2)-1
          do i=0,nn
             if( i .lt. int(0.5*alpha*float(nn)) )then
                temp(1+i) = 0.5 + 0.5*cos(pi*(2.0*float(i)/(alpha*float(nn)) - 1.0 ))
             elseif( i.ge.int(0.5*alpha*float(nn)) .and. i.le.(nn-int(0.5*alpha*float(nn))) )then
                temp(1+i) = 1.
             else
                temp(1+i) = 0.5 + 0.5*cos(pi*(2.0*float(i)/(alpha*float(nn)) - 2.0/alpha + 1.0 ))
             end if
          end do
          temp(n/2+1:n) = 0.
       end if
    end if

  end subroutine filter1Dfn

  subroutine filter2Dfn(temp,mode,flag,phi)
    !! mode=0 -- do nothing
    !! mode=1 -- sharp cut
    !! mode=2 -- Tukey filter using alpha parameter
    real,dimension(:,:),intent(inout) :: temp
    integer                           :: mode, flag, n1, n2, nn1,nn2, i,j
    real,intent(inout),optional          :: phi  ! input is degree, output=radian

    if( .not.present(phi) .and. mode.eq.2) then
       print *, "WARNING: phi is not present when using mode2 in filter2Dfn()"
       phi=30.  ! 30' is recommended
    end if
    phi = phi*pi/180.  ! convert from degree to radian
    
    !! temp(n1/2+1,n2/2+1) is always at zero frequency or wavenumber
    n1=size(temp,1)
    n2=size(temp,2)
    !write(0,*) "Hey! we are in filter2Dfn****"
    !write(0,*) "n1,n2,size(temp),mode,flag",n1,n2,size(temp),mode,flag
    
    if(mode.eq.0) then ! do nothing
       temp = 1.
    end if
    
    if(mode.eq.1) then ! step cut
       if(flag.eq.1) then
          ! flag=1: (+,+)axes and (-,-)axes will be stored, (+,-)axes and (-,+)axes will be removed.
          temp(n1/2+1:n1,n2/2+1:n2) = 1.  ! (+,+)
          temp(n1/2+1:n1,1:n2/2) = 0.     ! (+,-)
          temp(1:n1/2,n2/2+1:n2) = 0.     ! (-,+)
          temp(1:n1/2,1:n2/2) = 1.        ! (-,-)
        
       else
          ! flag=-1: (+,-)axes and (-,+)axes will be stored, (+,+)axes and (-,-)axes will be removed.
          temp(n1/2+1:n1,n2/2+1:n2) = 0.  ! (+,+)
          temp(n1/2+1:n1,1:n2/2) = 1.     ! (+,-)
          temp(1:n1/2,n2/2+1:n2) = 1.     ! (-,+)
          temp(1:n1/2,1:n2/2) = 0.        ! (-,-)
       end if       
    end if

    if(mode.eq.2)then ! Fan with Tukey tapering function

       if(flag.eq.1) then
          ! flag=1: (+,+)axes and (-,-)axes will be stored, (+,-)axes and (-,+)axes will be removed.

          temp(n1/2+1:n1,n2/2+1:n2) = 1.  ! (+,+) => with fan taper
          temp(n1/2+1:n1,1:n2/2) = 0.     ! (+,-)
          temp(1:n1/2,n2/2+1:n2) = 0.     ! (-,+)
          temp(1:n1/2,1:n2/2) = 1.        ! (-,-) => with fan taper

          nn1=n1-int(n1/2)-1 ! for positive elements on the 1st axis
          nn2=n2-int(n2/2)-1 ! for positive elements on the 2st axis
          !! apply fan taper at the 1st axis (+)
          do i=0,nn1  ! or i=0,nn1
             do j=0,int(float(i)*tan(phi))  ! 
                if( float(j) .lt. float(i)*tan(phi) )then
                   temp(n1/2+1+i,n2/2+1+j) = 0.5 + 0.5*cos(pi*(float(j)/(float(i)*tan(phi)) - 1.0 ))
                end if
             end do
          end do
          !! apply fan taper at the 2st axis (+)
          do j=0,nn2
             do i=0,int(float(j)*tan(phi))  ! 
                if( float(i) .lt. float(j)*tan(phi) )then
                   temp(n1/2+1+i,n2/2+1+j) = 0.5 + 0.5*cos(pi*(float(i)/(float(j)*tan(phi)) - 1.0 ))
                end if
             end do
          end do

          nn1=int(n1/2)-1 ! for negative elements on the 1st axis
          nn2=int(n2/2)-1 ! for negative elements on the 2st axis
          !! apply fan taper at the 1st axis (-)
          do i=0,nn1
             do j=0,int(float(i)*tan(phi))  ! 
                if( float(j) .lt. float(i)*tan(phi) )then
                   temp(n1/2-i,n2/2-j) = 0.5 + 0.5*cos(pi*(float(j)/(float(i)*tan(phi)) - 1.0 ))
                end if
             end do
          end do
          !! apply fan taper at the 2st axis (-)
          do j=0,nn2
             do i=0,int(float(j)*tan(phi))  ! 
                if( float(i) .lt. float(j)*tan(phi) )then
                   temp(n1/2-i,n2/2-j) = 0.5 + 0.5*cos(pi*(float(i)/(float(j)*tan(phi)) - 1.0 ))
                end if
             end do
          end do
        
       else
          ! flag=-1: (+,-)axes and (-,+)axes will be stored, (+,+)axes and (-,-)axes will be removed.
          temp(n1/2+1:n1,n2/2+1:n2) = 0.  ! (+,+)
          temp(n1/2+1:n1,1:n2/2) = 1.     ! (+,-) => with fan taper
          temp(1:n1/2,n2/2+1:n2) = 1.     ! (-,+) => with fan taper
          temp(1:n1/2,1:n2/2) = 0.        ! (-,-)


          !! (+,-) fan taper
          nn1=n1-int(n1/2)-1 ! for positive elements on the 1st axis
          nn2=int(n2/2)-1 ! for negative elements on the 2st axis
          !! apply fan taper at the 1st axis (+)
          do i=0,nn1  ! or i=0,nn1
             do j=0,int(float(i)*tan(phi))  ! 
                if( float(j) .lt. float(i)*tan(phi) )then
                   temp(n1/2+1+i,n2/2-j) = 0.5 + 0.5*cos(pi*(float(j)/(float(i)*tan(phi)) - 1.0 ))
                end if
             end do
          end do
          !! apply fan taper at the 2st axis (-)
          do j=0,nn2
             do i=0,int(float(j)*tan(phi))  ! 
                if( float(i) .lt. float(j)*tan(phi) )then
                   temp(n1/2+1+i,n2/2-j) = 0.5 + 0.5*cos(pi*(float(i)/(float(j)*tan(phi)) - 1.0 ))
                end if
             end do
          end do

          !! (-,+) fan taper
          nn1=int(n1/2)-1 ! for positive elements on the 1st axis
          nn2=n2-int(n2/2)-1 ! for negative elements on the 2st axis
          !! apply fan taper at the 1st axis (+)
          do i=0,nn1  ! or i=0,nn1
             do j=0,int(float(i)*tan(phi))  ! 
                if( float(j) .lt. float(i)*tan(phi) )then
                   temp(n1/2-i,n2/2+1+j) = 0.5 + 0.5*cos(pi*(float(j)/(float(i)*tan(phi)) - 1.0 ))
                end if
             end do
          end do
          !! apply fan taper at the 2st axis (-)
          do j=0,nn2
             do i=0,int(float(j)*tan(phi))  ! 
                if( float(i) .lt. float(j)*tan(phi) )then
                   temp(n1/2-i,n2/2+1+j) = 0.5 + 0.5*cos(pi*(float(i)/(float(j)*tan(phi)) - 1.0 ))
                end if
             end do
          end do

       end if       
    end if



    
  end subroutine filter2Dfn


end module
